#include "control.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "configuration.h"

Configuration::Configuration(ReadTextFile* confF ): confFile(confF),
		tempIs(false), uninstIs(false), pathIs(false),buffIs(false),
		packNameIs(false), pathNum(0), curPathSize(0)
{
	setDefault();
	
	errorNum = runInit();
	return;
}

Configuration::~Configuration()
{
	if(tempIs) free(tempPoint);
	if(uninstIs) free(uninstPoint);
	if(buffIs) free(pathBuff);
	if(packNameIs) free(packName);
	
	while(curPathSize){
		curPathSize--;
#ifdef _DEBUG
		printf("Del> %d\n", curPathSize);
#endif
		free(pathPoint[curPathSize]);
	}
		
	if(pathIs) free(pathPoint);
#ifdef _DEBUG
	printf("Deleting ConfObject...\n");
#endif
}

char* Configuration::getAlias(const char* nameFile)
{
	char* Tname = (char*)calloc( sizeof(char), strlen(nameFile)+2 );
	assert( Tname != NULL );
	char* pBuf = pathBuff;
	
	strcpy( Tname+1, nameFile);
	*(Tname)='@';
#ifdef _DEBUG
	printf("GetAlias: %s\tnameFile: %s\n", Tname, nameFile);
#endif
	confFile->goToStart();

	while(! confFile->eof() ){
		strcpy( pBuf, confFile->readLine() );
		if(!strncmp( pBuf, Tname, strlen(Tname))){
			pBuf += (strlen(Tname)+1);
			if(*(pBuf-1) != '='){	// '=' is missing after alias name
				errorNum = 4;
				free(Tname);
				return( (char*)NULL );
			}
			pBuf += ReadTextFile::excludeSpace(pBuf);
			errorNum = 0;
			if(*pBuf != '/'){	//each path in alias must starts with '/'
				errorNum = 14;
			}
			delEof(pBuf);
			free(Tname);
			return(pBuf);
		}
	}
#ifdef _DEBUG
	printf("No Match!\n");
#endif
	errorNum = 4;
						//there isn't that alias name!
	free(Tname);
	return( (char*)NULL );

}


const char* Configuration::getPath()
{
	if(!eop()){
		errorNum=0;
		return(pathPoint[pathNum++]);
	}
	else{
		errorNum=7;	//out of pathList
		return( (char*)NULL);
	}
}

bool Configuration::eop() const
{
	if((curPathSize-1)<pathNum) return(true);
	else return(false);
}

void Configuration::setName(const char* tmp)
{
	char* tP = (char*)calloc( sizeof(char), strlen(tmp)+1 );
	assert( tP != NULL );
	(void) strcpy( tP, tmp);
	delEof(tP);
#ifdef _DEBUG
	printf("setName: charLen: %d\n", strlen(tP));
#endif
	if(!strlen(tP)){
		free(tP);
		return;
	}
	
	packName = tP;
	packNameIs = true;

}

void Configuration::setTemp(const char* tmp)
{
	char* tP = (char*)calloc( sizeof(char), strlen(tmp)+1 );
	assert( tP != NULL );
	(void) strcpy( tP, tmp);
	delEof(tP);
#ifdef _DEBUG
	printf("setTemp: charLen: %d\n", strlen(tP));
#endif
	if(!strlen(tP)){
		free(tP);
		return;
	}
	
	tempPoint = tP;
	tempIs = true;
}

void Configuration::setUninst(const char* uni)
{
	char* tP = (char*)calloc( sizeof(char), strlen(uni)+1 );
	assert( tP != NULL );
	(void) strcpy( tP, uni);
	delEof(tP);
#ifdef _DEBUG
	printf("setUninst: charLen: %d\n", strlen(tP));
#endif
	if(!strlen(tP)){
		free(tP);
		return;
	}
	
	uninstPoint = tP;
	uninstIs = true;
}


void Configuration::writeBeforeFile(WriteTextFile* beforeFile)
{
	beforeFile->writeLine("#beforeFile for Uninst " VERSION "\nFLAGS= ");
	char* flag;
	if(getForce()==SET_ON) flag="-f ";
	else flag="-nf ";
	beforeFile->writeLine(flag);
	if(getWrite()==SET_ON) flag="-v ";
	else flag="-nv ";
	beforeFile->writeLine(flag);
	if(getNoRemove()==SET_ON) flag="-nr ";
	else flag="-r ";
	beforeFile->writeLine(flag);
	
	beforeFile->writeLine("\nTEMP=");
	beforeFile->writeLine(getTemp());
	beforeFile->writeLine("\nUNINSTAL=");
	beforeFile->writeLine(getUninst());
	beforeFile->writeLine("\nPATHNUM=");
	char pathString[10];
	sprintf(pathString,"%d", getPathSize());
	beforeFile->writeLine(pathString);
	beforeFile->writeLine("\nNAME=");
	beforeFile->writeLine(getPackName());
	beforeFile->writeLine("\n--paths---\n");
	setPathNum(0);	//go to start of path-array
	while(!eop()){
		beforeFile->writeLine(getPath());
		beforeFile->writeLine("\n");
	}
	beforeFile->writeLine("---files---\n");
}

void Configuration::PathToPath(const char* string)
{
	if(!strcmp(string, "/")){
		errorNum=8;
		return;		//can't search whole filesystem!
	}
	if((curPathSize+2)==pathSize){
		errorNum=98;		//out of path memory!
		return;
	}
	pathPoint[curPathSize] = (char*)calloc( sizeof(char), strlen(string)+1);
	assert( pathPoint[curPathSize]!=NULL);
	strcpy( pathPoint[curPathSize], string);
	if(*(pathPoint[curPathSize]+strlen(pathPoint[curPathSize])-1)=='/'){
		*(pathPoint[curPathSize]+strlen(pathPoint[curPathSize])-1)='\0';
#ifdef _DEBUG
		printf("slash correction\n");
		printf("PathToPath: %s\n", pathPoint[curPathSize]);
#endif
	}	
	curPathSize++;
	errorNum=0;
	return;
}

void Configuration::AliasToPath(const char* string)
{
	char* line = getAlias( string);
	if(errorNum){
		return;
	}
	if(*line=='\0') return;		//alias is empty
	int x=0;
	while(1){
		if(line[x]=='\0'){
			if(line[x-1]=='\0') break;
#ifdef _DEBUG
			printf("DDAliasToAlias: %s\n", line);
#endif
			PathToPath(line);
			if(errorNum){
				return;
			}
			break;
		}
		if(line[x]==':'){
			line[x]='\0';
			PathToPath(line);
#ifdef _DEBUG
			printf("AliasToAlias: %s\n", line);
#endif
			if(errorNum){
				return;
			}
			line+=(x+1);
			x=0;
		}
		else x++;
	}
	errorNum=0;
	return;
}

void Configuration::readBeforeFile(ReadTextFile* beforeFile)
{
	errorNum = 17;
	if( readConf( beforeFile, SET_AFTER ) ){
		return;
	}
	if( addBeforePaths( beforeFile ) ){
		return;
	}
#ifdef _DEBUG
	printf("==readBeforeFile -- OK\n");
#endif
	errorNum = 0;
}		

int Configuration::createConfFile(void)
{
	char homeDir[SIZE_BUFF];
	assert( homeDir != NULL );
	char* homeAdr;
	
	if( (homeAdr = getenv("HOME")) == NULL){	//get name of user HOME dir
		return(5);		//enviroment error
	}
	
	(void) strcpy( homeDir, homeAdr);
#ifdef _DEBUG
	printf("createConfFile: HOME = %s\n", homeDir);
#endif
	(void) strcat( homeDir, "/" CFIL );
	FILE* fw = fopen( homeDir, "w");
	if( fw == NULL ){
		return(9);		// unable to open(create) file
	}
	fprintf(fw,			// write default settings
			"# Uninst Configuration File\n\n"
			"FLAGS=\n"
			"TEMP=\n"
			"UNINSTAL=\n\n"
			"# Aliases\n"
			"@kde=/opt/kde\n"
			"@usr=/usr\n"
			"@all=/opt/kde:/usr:/bin:/sbin:/etc:/var\n"
		);
	if( fclose(fw) ){
#ifdef _DEBUG
		perror("fclose");
#endif
		return(10);
	}		
	return(0);
}

void Configuration::setDefault()
{
	forceFlag=SET_ON;
	writeFlag=SET_OFF;
	noRemoveFlag=SET_OFF;
	tempPoint="/tmp";
	tempIs=false;
	uninstPoint=".";
	uninstIs=false;
	pathSize=PATH_SIZE;
	packName="Program";
	
}

int Configuration::runInit()
{
	int x;
	if( (x = readConf( confFile )) ){
		return(x);
	}
	pathPoint = (char**)calloc( sizeof(char*), getPathSize());
	assert( pathPoint != NULL );
	pathIs = true;
	pathBuff = (char*)calloc( sizeof(char), SIZE_BUFF);
	assert( pathBuff != NULL );
	buffIs = true;
	return(0);
		
}

int Configuration::readConf(ReadTextFile* fr, char endT)
{
	fr->goToStart();
	const char* tmpC;
	
	while(!fr->eof()){
		tmpC = fr->readLine();
		if((*tmpC=='#') || (*tmpC=='\n') || (*tmpC=='@')) continue;
		
		if( endT==SET_AFTER && (!strncmp( tmpC, "NAME=", 5)) ){
			setName(tmpC+5);
			break;
		}
					
		if(!strncmp( tmpC,"FLAGS=", 6)){
			if(setFlags(tmpC+6)){
				return(10);	// syntax error or missing parameter in confFile
			}
			continue;
		}
		
		if(!strncmp( tmpC, "TEMP=", 5)){
			setTemp(tmpC+5);
			continue;
		}
		
		if(!strncmp( tmpC, "UNINSTAL=", 9)){
			setUninst( tmpC+9);
			continue;
		}
		
		if(!strncmp( tmpC, "PATHNUM=", 8)){
			if(setPathSize(tmpC+8)){
				return(10);
			}
			continue;
		}
		return(11);		// unknown parameter or syntax error in confFile
	}
	return(0);
}

int Configuration::setFlags(const char* line)
{
	line += ReadTextFile::excludeSpace( line );
#ifdef _DEBUG
	printf("setFlag: >>%s<<\n", line);
#endif	
	while( ((*line)!='\n') && ((*line)!='\0') ){

		if( ((*line)==' ') || ((*line)=='\t') ){
			line+=1;
			continue;
		}
		if(!strncmp( line,"-f", 2)){
			setForce(SET_ON);
			line+=2;
			continue;
		}
		if(!strncmp( line, "--force", 7)){
			setForce(SET_ON);
			line+=7;
			continue;
		}
		if(!strncmp( line, "-v", 2)){
			setWrite(SET_ON);
			line+=2;
			continue;
		}
		if(!strncmp( line, "--verbose", 9)){
			setWrite(SET_ON);
			line+=9;
			continue;
		}
		if(!strncmp( line, "-nr", 3)){
			setNoRemove(SET_ON);
			line+=3;
			continue;
		}
		if(!strncmp( line, "--no-remove", 11)){
			setNoRemove(SET_ON);
			line+=11;
			continue;
		}
		if(!strncmp( line, "--no-force", 10)){
			setForce(SET_OFF);
			line+=10;
			continue;
		}
		if(!strncmp( line, "-nf", 3)){
			setForce(SET_OFF);
			line+=3;
			continue;
		}
		if(!strncmp( line, "-nv", 3)){
			setWrite(SET_OFF);
			line+=3;
			continue;
		}
		if(!strncmp( line, "--no-verbose", 12)){
			setWrite(SET_OFF);
			line+=12;
			continue;
		}
		if(!strncmp( line, "-r", 2)){
			setNoRemove(SET_OFF);
			line+=2;
			continue;
		}
		if(!strncmp( line, "--remove", 8)){
			setNoRemove(SET_OFF);
			line+=8;
			continue;
		}
			
#ifdef _DEBUG
		printf("Error StrinG: %s\n", line);
#endif
		return(10);		// syntax error!
	}
	return(0);
}

int Configuration::setPathSize(const char* line)
{	
	int x;
	int y = sscanf( line, " %d", &x );
	switch(y){
		case EOF:
			return(0);
		case 0:
			return(10);
		default:
			if( (x > 299) && (x < 10000) ){	//pathNum must be between
				pathSize=x;
				return(0);
			}
#ifdef _DEBUG
			printf("--pathSIZE= %d\tret: %d\n", x, y);
#endif
			return(0);
	}
}

void Configuration::delEof( char* tmp)
{
	int x=strlen(tmp)-1;
#ifdef _DEBUG
	printf("delEof:\tstrlen1= %d\n", x);
#endif
	while( ( x >= 0 ) && ((tmp[x]=='\n') || (tmp[x]==' ') || (tmp[x]=='\t')) ){
		x--;
	}
#ifdef _DEBUG
	printf("delEof:\tstrlen2= %d\n", x);
#endif
	*(tmp+x+1)='\0';
	
}

int Configuration::addBeforePaths(ReadTextFile* beforeFile)
{
	if(strncmp(beforeFile->readLine(), "--paths---", 10)){
		return(17);	//error in beforeFile
	}
	char temp[SIZE_BUFF];
	
	while(strncmp((strncpy( temp, beforeFile->readLine(), SIZE_BUFF)),"---files---", 11)){
		delEof(temp);
		PathToPath(temp);
		if(errorNum){
			return(errorNum);
		}
	}
	return(0);
}

