// -*- c++ -*-
#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "control.h"
#include <errno.h>
#include "writetextfile.h"
#include "readtextfile.h"

// from ver. 0.1.6 configuration file is named ".uninstrc" and it's
// located in home directory
#define CFIL ".uninstrc"		// confDir

extern int errno;

class Configuration
{
	public:
		Configuration(ReadTextFile* confFile);
		~Configuration();
	
		//static
		static int createConfFile(void);
	
		//both
		char getForce() const
		{
			return(forceFlag);
		}
		char getWrite() const
		{
			return(writeFlag);
		}
		char getNoRemove() const
		{
			return(noRemoveFlag);
		}
		int getPathNum() const
		{
			return(pathNum);
		}
		int getPathSize() const
		{
			return(pathSize);
		}
		int getCurPathSize() const
		{
			return(curPathSize);
		}
		const char* getPackName() const
		{
			return(packName);
		}
		const char* getTemp() const
		{
			return(tempPoint);
		}
		const char* getUninst()const
		{
			return(uninstPoint);
		}
		int errorCode() const
		{
			return(errorNum);
		}
		
		char* getAlias(const char*);
		const char* getPath();
		bool eop() const;
		
		//before
		void setForce(char par)
		{
			forceFlag=par;
		}
		void setWrite(char par)
		{
			writeFlag=par;
		}
		void setNoRemove(char par)
		{
			noRemoveFlag=par;
		}
		void setPathNum(int par)
		{
			pathNum=par;
		}
		void pointName(char* name)
		{
			packName = name;
		}
		
		void setTemp(const char*);
		void setUninst(const char*);
		void setName(const char*);
		void writeBeforeFile(WriteTextFile* beforeFile);
		void PathToPath(const char*);
		void AliasToPath(const char*);
	
		//after
		void readBeforeFile(ReadTextFile* beforeFile);

	private:
		ReadTextFile* confFile;
		WriteTextFile* beforeFile;
		
		char forceFlag;
		char writeFlag;
		char noRemoveFlag;
		
		char* tempPoint;	bool tempIs;
		char* uninstPoint;	bool uninstIs;
		char** pathPoint;	bool pathIs;
		char* pathBuff;		bool buffIs;
		char* packName;		bool packNameIs;	//name of uninst script
		
		int pathNum;            //'pointer' to another path to read
		int pathSize;		//max. number of paths (300=default)
		int curPathSize; 	//current max. path number
		
		int errorNum;
		
		void setDefault();
		int runInit();
		int readConf(ReadTextFile* fr, char endT = SET_BEFORE);
		int setFlags(const char* line);
		int setPathSize(const char* line);
		int addBeforePaths(ReadTextFile* beforeFile);
		static void delEof( char* tmp);
};

#endif // CONFIGURATION_H

