#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "readtextfile.h"

ReadTextFile::ReadTextFile(const char* nameString, bool makePipe):
				eofCode(false),inUse(false), buffIs(false),
				nameIs(false), pipeIs(makePipe), posF(0),
				posBreak(0)
{
 	if( nameString == 0 ){
 		errorNum=16;		// pointer is invalid
		return;
	}
		
	fileName = (char*)calloc( sizeof(char), strlen( nameString) + 1);
	if( fileName == NULL){
		errorNum=3;		// memory error;
		return;
	}
	nameIs=true;
	
	(void) strcpy( fileName, nameString);
#ifdef _DEBUG
	printf("ReadTextFile: file = %s\n", fileName);
#endif
	if(pipeIs){
		fr = popen( fileName, "r");
		if( fr == NULL ){
			errorNum=16;		// unable to open pipe
			return;
		}
	}	
	 else{
		fr = fopen( fileName, "r");
		if( fr == NULL ){
			errorNum=16;		// unable to open file
			return;
	  	}
	 }	
	inUse=true;
	
	charBuff = (char*)calloc( sizeof(char), SIZE_BUFF );
	if( charBuff == NULL ){
		errorNum=3;		// memory error
		return;
	}
	buffIs=true;
	
	errorNum=0;			// OK!
	
}

ReadTextFile::~ReadTextFile()
{
	if(inUse){
		if(pipeIs){
			(void)pclose(fr);
		}
		 else{
		 	(void)fclose(fr);
		 }
	}
	
	if(nameIs) free(fileName);
	if(buffIs) free(charBuff);
#ifdef _DEBUG
	printf("~ReadTextFile: Deleting object...\n");
#endif
}

const char* ReadTextFile::readLine(bool exSpa)
{
	if( (fgets( charBuff, SIZE_BUFF, fr )) == NULL ){
		errorNum=2;
		eofCode=true;	// end of file
		return("");
	}
	
	int test;
	if((test = getc(fr)) == EOF){
		errorNum=2;
		eofCode=true;	// end of file
	}
	ungetc(test, fr);
 	errorNum=0;
	
	if(exSpa==true){
		return(charBuff + excludeSpace(charBuff));
	}
	  else{
		return(charBuff);
	  }
}

void ReadTextFile::close()
{
	if(inUse==false){
		errorNum=5;
		return;
	}

	if(fr == NULL){
		errorNum=5;
		return;
	}
	
	if(pipeIs){
		if(pclose(fr) == -1){
			errorNum=5;	// unable to close pipe
			return;
		}
	}
	 else{
	 	if(fclose(fr) == EOF){
		 	errorNum=5;	// unable to close file
		 	return;
		}
	 }
	
	fr=NULL;
	inUse=false;
	errorNum=0;
#ifdef _DEBUG
	printf("ReadTextFile::close:	File closed...\n");
#endif
	return;
}

void ReadTextFile::deleteFile()
{
	(void) fclose(fr);
	
	fr=NULL;
	inUse=false;
	
	if( (remove( fileName )) == -1){
		errorNum=8;		// can't delete file
		return;
	}
	errorNum=0;
	return;
}

int ReadTextFile::excludeSpace(const char* fileLine)
{
	int x=0;
	while( fileLine[x]=='\t' || fileLine[x]==0x20 ){
		x++;
	}
	return(x);
}

void ReadTextFile::goToStart()
{
	if(inUse){
/*#ifdef _DEBUG
		printf("Rewind\n");
#endif*/
		fseek ( fr, posF, SEEK_SET );
		eofCode=false;
	}
}

void ReadTextFile::goToBreak()
{
	if(inUse){
/*#ifdef _DEBUG
		printf("Rewind break\n");
#endif*/
		fseek ( fr, posBreak, SEEK_SET );
		eofCode=false;
	}
}

const char* ReadTextFile::getName() const
{
	if(nameIs){
		return(fileName);
	}
	return((char*)NULL);
}

void ReadTextFile::setStartToCur()
{
	posF = ftell (fr);
/*#ifdef _DEBUG
	printf("|POS| %ld\n", posF);
#endif*/
}

void ReadTextFile::setBreakToCur()
{
	posBreak = ftell (fr);
/*#ifdef _DEBUG
	printf("|POS_BREAK| %ld\n", posBreak);
#endif*/
}

long int ReadTextFile::getPos()
{
	return( ftell(fr) );
}

void ReadTextFile::setPos(long int cPos)
{
	if(inUse){
		fseek ( fr, cPos, SEEK_SET );
		eofCode=false;
	}

}
