#include "uninst.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

Uninst::Uninst(int param1, char **param2): argc(param1), argv(param2), homeIs(false), fileIs(false),
					confIs(false), beforeIs(false), afterIs(false), AbeforeIs(false),
					AafterIs(false), uninstIs(false)
{
	if(argc<2){			//parameter missing!
		errorNum = 1;
		return;
	}
	
#ifdef _DEBUG
	printf("Arg1: %s\n", argv[1]);
#endif
	if( (!strcmp( argv[1], "--help")) || (!strcmp( argv[1], "-h")) || (!strcmp( argv[1], "h"))){
		errorNum = 500;
		return;
	}
	if( (!strcmp( argv[1], "-b")) || (!strcmp( argv[1], "--before")) || (!strcmp( argv[1], "b")) ){
		mode = SET_BEFORE;
	}
	else if( (!strcmp( argv[1], "-a")) || (!strcmp( argv[1], "--after")) || (!strcmp( argv[1], "a")) ){
		mode = SET_AFTER;
		if(argc>2){
			printf("\"%s\" - ", argv[2]);
			errorNum = 6;		//invaid parameter
			return;
		}
	}
	else{
		printf("\"%s\" - ", argv[1]);
		errorNum = 6;			//invalid parameter
		return;
	}
		
	
	homeDir = (char*)calloc( sizeof(char),SIZE_BUFF );
	assert( homeDir != NULL );
	homeIs=true;
	
	
	char* homeAdr;
	
	if( (homeAdr = getenv("HOME")) == NULL){	//get name of user HOME dir
		errorNum = 5;		//enviroment error
		return;
	}
	
	(void) strcpy( homeDir, homeAdr);
	(void) strcat( homeDir, "/" CFIL );
	
	file = new ReadTextFile(homeDir);		//read confFile
	if(file->errorCode()){
		delete file;
		int x;
		if( (x = Configuration::createConfFile()) ){
			errorNum = 99;
			return;
		}
		file = new ReadTextFile(homeDir);
		if(file->errorCode()){
			delete file;
			errorNum = 12;
			return;
		}
	}
	fileIs = true;
	
	conf = new Configuration(file);
	confIs = true;
	int x;
	if( (x = conf->errorCode()) ){
			errorNum = x;
			return;
	}
	errorNum = 0;
}

Uninst::~Uninst()
{
	if(beforeIs) delete beforeFile;
	if(afterIs) delete afterFile;
	if(AbeforeIs) delete AbeforeFile;
	if(AafterIs) delete AafterFile;
	if(uninstIs) delete uninstFile;
	if(confIs) delete conf;
	if(fileIs) delete file;
	if(homeIs) free(homeDir);
#ifdef _DEBUG
	printf("~Uninst: deleting object...\n");
#endif
}

int Uninst::exec()
{
	if(errorNum){			//error in constructor!
#ifdef _DEBUG
		printf("Error in Uninst constructor!\n");
#endif
		if(errorNum==500){
			Messages::printMessage(499);
		}	
		Messages::printMessage(errorNum);
		return((errorNum==500) ? 0 : errorNum);
	}
	
	assert((mode==SET_BEFORE) || (mode==SET_AFTER));
	
	switch(mode){
		case SET_BEFORE:
			procBefore();
#ifdef _DEBUG
			printf("Force:\t%d\n", conf->getForce());
			printf("Verbose:\t%d\n", conf->getWrite());
			printf("NoRemove:\t%d\n", conf->getNoRemove());
			printf("PathSize:\t%d\n", conf->getPathSize());
			printf("TEMP:\t%s\n", conf->getTemp());
			printf("UNINST:\t%s\n", conf->getUninst());	
			printf("OUTPUT:\t%s\n", conf->getPackName());
			conf->setPathNum(0);
			while(!conf->eop()){
				printf("# %s\n", conf->getPath());
			}
#endif
                        break;
		
		case SET_AFTER:
			procAfter();
#ifdef _DEBUG
			printf("Force:\t%d\n", conf->getForce());
			printf("Verbose:\t%d\n", conf->getWrite());
			printf("NoRemove:\t%d\n", conf->getNoRemove());
			printf("PathSize:\t%d\n", conf->getPathSize());
			printf("TEMP:\t%s\n", conf->getTemp());
			printf("UNINST:\t%s\n", conf->getUninst());	
			printf("OUTPUT:\t%s\n", conf->getPackName());
			conf->setPathNum(0);
			while(!conf->eop()){
				printf("# %s\n", conf->getPath());
			}
#endif

			break;
	}
	return((errorNum==500) ? 0 : errorNum);

}

void Uninst::procBefore()
{
	if( (errorNum = setArg()) ){
		Messages::printMessage(errorNum);
		return;
	}
	if( (errorNum = makeBeforeFile()) ){
		Messages::printMessage(errorNum);
	        return;
	}
        if( (errorNum = listFiles(beforeFile)) ){
		if(errorNum==19){	// delete beforeFile if there was no output from 'ls'
	        	beforeFile->deleteFile();
	 	}	
	        Messages::printMessage(errorNum);
	        return;
	}
	if(conf->getWrite()==SET_ON){
		printf(" %s", beforeFile->getName());
		Messages::printMessage(502);
	}
}

void Uninst::procAfter()
{
	if( (errorNum = openBeforeFile()) ){
	             Messages::printMessage(errorNum);
	             return;
	}
	AbeforeFile->setStartToCur();
	
	if( (errorNum = makeAfterFile()) ){
	             Messages::printMessage(errorNum);
	             return;
	}
        if( (errorNum = listFiles(afterFile)) ){
	             Messages::printMessage(errorNum);
	             return;
	}
	if(conf->getWrite()==SET_ON){
		printf(" %s", afterFile->getName());
		Messages::printMessage(502);
	}
	
	delete afterFile;
	afterIs = false;
	
	if( (errorNum = openAfterFile()) ){
	             Messages::printMessage(errorNum);
	             return;
	}
	if( (errorNum = makeUninstFile()) ){
	             Messages::printMessage(errorNum);
	             return;
	}
	if(conf->getWrite()==SET_ON){
		printf(" %s -> %s", AbeforeFile->getName(), AafterFile->getName());
		Messages::printMessage(507);
	}
	
	if( (errorNum = compareFiles()) ){	//compare AbeforeFile & AafterFile
		Messages::printMessage(errorNum);
		if(errorNum == 20){	//there are no new files or dirs
	        	uninstFile->deleteFile();
		}
		 else{	
			return;			//some other error
		 }
	}
         else{
		uninstFile->writeLine(Messages::writeSome(3));	//########################\n
		uninstFile->writeLine("\necho \"");
		uninstFile->writeLine(Messages::writeSome(4));
		uninstFile->writeLine("\"\nelse\necho \"");
		uninstFile->writeLine(Messages::writeSome(5));
		uninstFile->writeLine("\"\nfi\n");

	        if(conf->getWrite()==SET_ON){
			printf(" %s", uninstFile->getName());
			Messages::printMessage(502);
		}
	 }
	
	if(conf->getNoRemove()==SET_OFF){
		AafterFile->deleteFile();
		if(conf->getWrite()==SET_ON){
			printf(" %s", AafterFile->getName());
			Messages::printMessage(509);
		}
		AbeforeFile->deleteFile();
		if(conf->getWrite()==SET_ON){
			printf(" %s", AbeforeFile->getName());
			Messages::printMessage(509);
		}
	}
}

int Uninst::setArg()
{
	for(int i=2; i<argc; i++ ){	//cycle trough all command parameters

#ifdef _DEBUG
		printf("|+|%d: %s\n", i, argv[i]);
#endif
		
		if( (!strcmp( argv[i], "--help")) || (!strcmp( argv[i], "-h")) ){	
			return(500);	//help
		}			
		
		if(*argv[i] == '@'){	//add alias
			if(strlen(argv[i]) > 1){
				conf->AliasToPath(argv[i]+1);
				if(conf->errorCode()){
					printf("\"%s\" - ", argv[i]);
					return(conf->errorCode());
				}
				continue;
			}
			printf("\"@\" - ");
			return(6);	//invalid parameter
		}
		
		if( (!strcmp( argv[i], "-f")) || (!strcmp( argv[i], "--force")) ){
			conf->setForce(SET_ON);
			continue;
		}
		if( (!strcmp( argv[i], "-v")) || (!strcmp( argv[i], "--verbose")) ){
			conf->setWrite(SET_ON);
			continue;
		}
		if( (!strcmp( argv[i], "-r")) || (!strcmp( argv[i], "--remove")) ){
			conf->setNoRemove(SET_OFF);
			continue;
		}
		if( (!strcmp( argv[i], "-nr")) || (!strcmp( argv[i], "--no-remove")) ){
			conf->setNoRemove(SET_ON);
			continue;
		}
		if( (!strcmp( argv[i], "-nf")) || (!strcmp( argv[i], "--no-force")) ){
			conf->setForce(SET_OFF);
			continue;
		}
		if( (!strcmp( argv[i], "-nv")) || (!strcmp( argv[i], "--no-verbose")) ){
			conf->setWrite(SET_OFF);
			continue;
		}
		
		
		if(!strcmp( argv[i], "-o")){	//'-o' was last parameter
			if(i==(argc-1)){
				return(7);
			}
			i++;
			conf->pointName(argv[i]);	//name of uninstall script
			continue;
		}
		
		if(*argv[i] == '/'){	//add path
			if(strlen(argv[i]) > 1){
				conf->PathToPath(argv[i]);
				if(conf->errorCode()){
					return(conf->errorCode());
				}
				continue;
			}
			return(8);	// can't search whole filesystem
		}
		
		printf("\"%s\" - ", argv[i]);
		return(6);	//invalid parameter
	}
	
	if(!conf->getCurPathSize()){
		return(9);	//no path were specified
	}
        return(0);
}

int Uninst::makeBeforeFile()		//make beforeFile (in '--before' mode)
{
	char tmpDir[SIZE_BUFF];
	int x;
	if( (x = setCName(tmpDir)) ){	//set name for beforeFile
		return(x);
	}
	(void) strcat( tmpDir, "-" CBIF );
#ifdef _DEBUG
	printf("makeBeforeFile: beforeFile = %s\n", tmpDir);
#endif
	if(checkExist(tmpDir)){		//check if file exists
		return(505);		//file exists and user do not want to overwrite it.
	}
		
	beforeFile = new WriteTextFile(tmpDir);	//create beforeFile
#ifdef _DEBUG
	printf("makeBeforeFile: ERROR = %d\n", beforeFile->errorCode());
#endif
	if( (x = beforeFile->errorCode()) ){
		delete beforeFile;
		printf("\"%s\" - ", tmpDir);
		return(x);
	}
	beforeIs = true;
	
	conf->writeBeforeFile(beforeFile);	//write confFile to beforeFile
	if(conf->errorCode()){
		printf("\"%s\" - ", tmpDir);
		return(conf->errorCode());
	}
	return(0);	
}

int Uninst::makeAfterFile()		//make afterFile (in '--after' mode)
{
	char tmpDir[SIZE_BUFF];
	int x;
	if( (x = setCName(tmpDir)) ){	//set name for beforeFile
		return(x);
	}
	(void) strcat( tmpDir, "-" CAFT );
#ifdef _DEBUG
	printf("makeAfterFile: afterFile = %s\n", tmpDir);
#endif
	if(checkExist(tmpDir)){		//check if file exists
		return(505);		//file exists and user do not want to overwrite it.
	}
		
	afterFile = new WriteTextFile(tmpDir);	//create AfterFile
#ifdef _DEBUG
	printf("makeAfterFile: ERROR = %d\n", afterFile->errorCode());
#endif
	if( (x = afterFile->errorCode()) ){
		delete afterFile;
		printf("\"%s\" - ", tmpDir);
		return(x);
	}
	afterIs = true;
	
	return(0);	
}

bool Uninst::checkExist(char* tmpDir ) //if returns true -> stop
{
	ReadTextFile* test = new ReadTextFile(tmpDir);	//check if file exists
	if(!test->errorCode()){
		printf(" %s", tmpDir);
		Messages::printMessage(504);
		printf("[Y/N]");
		int c;
		if( (c=fgetc(stdin)) == '\n'){
		 	delete test;
		 	return(true);
		}
		 else if( ( c != 'y' ) && ( c != 'Y' )){
			delete test;
			while( (c=fgetc(stdin))!='\n');
			return(true);
		 }
		  else{
			while( (c=fgetc(stdin))!='\n');
		  }
	}
	delete(test);
	    return(false);
}

int Uninst::openBeforeFile()		//open beforeFile (in '--after' mode)
{
	char tmpDir[SIZE_BUFF];
	int x;
	if( (x = setCName(tmpDir)) ){	//set name for beforeFile
		return(x);
	}
	(void) strcat( tmpDir, "-" CBIF );
#ifdef _DEBUG
	printf("openBeforeFile: beforeFile = %s\n", tmpDir);
#endif
	AbeforeFile = new ReadTextFile(tmpDir);	//open beforeFile
#ifdef _DEBUG
	printf("openBeforeFile: ERROR = %d\n", AbeforeFile->errorCode());
#endif
	if( (x = AbeforeFile->errorCode()) ){
		delete AbeforeFile;
		Messages::printMessage(503);
		printf("\"%s\" - ", tmpDir);
		return(x);
	}
	AbeforeIs = true;
	
	conf->readBeforeFile(AbeforeFile);	//read beforeFile as confFile
	if(conf->errorCode()){
		printf("\"%s\" - ", tmpDir);
		return(conf->errorCode());
	}
	return(0);	
}

int Uninst::openAfterFile()		//open afterFile (in 'after' mode) for
{					//comparing with beforeFile
	char tmpDir[SIZE_BUFF];
	int x;
	if( (x = setCName(tmpDir)) ){	//set name for afterFile
		return(x);
	}
	(void) strcat( tmpDir, "-" CAFT );
#ifdef _DEBUG
	printf("openAfterFile: AafterFile = %s\n", tmpDir);
#endif
	AafterFile = new ReadTextFile(tmpDir);	//open afterFile
#ifdef _DEBUG
	printf("openAfterFile: ERROR = %d\n", AafterFile->errorCode());
#endif
	if( (x = AafterFile->errorCode()) ){
		delete AafterFile;
		printf("\"%s\" - ", tmpDir);
		return(x);
	}
	AafterIs = true;
        return(0);
}

int Uninst::makeUninstFile()		//make uninstFile (uninstall script)
{
	char tmpDir[SIZE_BUFF];
	int x;
	(void) strcpy( tmpDir, conf->getUninst());
	(void) strcat( tmpDir, "/");
	(void) strcat( tmpDir, conf->getPackName());
#ifdef _DEBUG
	printf(")>UNINST = %s\n", tmpDir );
#endif
	if(checkExist(tmpDir)){		//check if file exists
		return(505);		//file exists and user do not want to overwrite it.
	}	

	uninstFile = new WriteTextFile(tmpDir);	//open uninstFile
#ifdef _DEBUG
	printf("openUnistFile: ERROR = %d\n", uninstFile->errorCode());
#endif
	if( (x = uninstFile->errorCode()) ){
		delete uninstFile;
		Messages::printMessage(506);
		printf("\"%s\" - ", conf->getUninst());
		return(x);
	}
	uninstIs = true;
        uninstFile->setRunBit();
#ifdef _DEBUG
        if(uninstFile->errorCode()){
        	printf("ooooooo SET_RUN_BIT fails!\n");
        }
#endif
        uninstFile->writeLine(
        "#!/bin/sh\n"
       	"# Generated automatically by Uninst " VERSION "\n\n"
//
//	add date and time
//
	"echo \"");
	uninstFile->writeLine(Messages::writeSome(1));	//are you sure you want to uninstall
	uninstFile->writeLine(" '");
	uninstFile->writeLine(conf->getPackName());
	uninstFile->writeLine(
	"' ?\"\n"
	"echo \"[Y/N]\"\n"
	"read YN\n"
	"if [ \"$YN\" = \"Y\" ] || [ \"$YN\" = \"y\" ]; then\n"
	"echo \"");
	uninstFile->writeLine(Messages::writeSome(2));	//uninstaling...
	uninstFile->writeLine("\"\n\n");
	uninstFile->writeLine(Messages::writeSome(3));	//####################################\n
	
        return(0);	
}

int Uninst::setCName(char* tmpDir) const
{
	char* logN;
	
	if( (logN = getenv("LOGNAME")) == NULL){	//get name user
		return(5);		//enviroment error
	}
	
	(void) strcpy( tmpDir, conf->getTemp());
	(void) strcat( tmpDir, "/");
	(void) strcat( tmpDir, logN);
	return(0);	
}

int Uninst::listFiles(WriteTextFile* procFile)
{
	char tmpChar[SIZE_BUFF+29];
	char tmpLine[SIZE_BUFF];
	bool checkLs = false;
#ifdef _DEBUG
	printf("*** listFiles ***\n");
#endif	
	strcpy( tmpChar , "/bin/ls -AFR1Q --color=none \"" );	// lenght=29
	
	conf->setPathNum(0);	//go to start of path names
	while(!conf->eop()){
			strcpy( tmpChar+29, conf->getPath() );
			if(conf->getWrite()==SET_ON){		// be verbose if --write is set
				printf(" %s", tmpChar+29 );
				Messages::printMessage(501);
			}
			strcat( tmpChar, "\"" );
#ifdef _DEBUG
			printf("*** listFiles *** -> %s\n", tmpChar);
#endif			
			ReadTextFile lsList( tmpChar, true);
			if(lsList.errorCode()){
				return(15);			// unable to create pipe
			}
			
			bool levelOne = true;
			while(!lsList.eof()){
				strcpy( tmpLine,lsList.readLine() );
				if(*tmpLine=='\0'){		//pipe is empty (path for 'ls' was wrong)
					conf->setPathNum( (conf->getPathNum())-1 );
					printf(" %s", conf->getPath());
					Messages::printMessage(508);
					break;
				}

				checkLs = true;		//results of 'ls' are not empty at all
				char* curLine = tmpLine;
				if(*curLine=='\n') continue;
				if(*curLine=='"'){
					curLine+=1;
					char* c = (curLine+strlen(curLine)-2);
					switch((int)*c){
						case '"':
							strcpy( c, "\n\0" );
							break;
						case '@':
						case '*':
							strcpy( c-1, "\n\0" );
							break;
						case ':':
							strcpy( c-1, "\n\0");
							levelOne = false;
							break;
							
						default: continue;
					}
					
					if(levelOne && (*curLine!='/')){
						levelOne = false;
						conf->setPathNum( (conf->getPathNum())-1 );
						procFile->writeLine( conf->getPath());
						procFile->writeLine("\n");
						procFile->writeLine( curLine );
					}
					 else{
					 	procFile->writeLine( curLine );
					 }
				}
				 else{
					levelOne = false;
					strcpy( (strrchr ( curLine, ':' )), "\n\0");
					procFile->writeLine( curLine );
				 }
				 		
			}
	}
	if(!checkLs) return(19);	//general result of 'ls' was empty (nothing to do)
	return(0);
}

int Uninst::compareFiles()
{
	AafterFile->goToStart();
	AbeforeFile->goToStart();
	
	const char* tmpLine;
	char tmpDir[SIZE_BUFF];
	strcpy(tmpDir, "-fatal-");
	
	bool changed = false;	//return false if there is no new files or dirs
	int partPos = 0;
	
	while(!AafterFile->eof()){
		tmpLine = AafterFile->readLine();
		
		if(*tmpLine=='/'){	//another dir
			partPos = 0;	//reset file pos. in new dir
			strcpy(tmpDir, tmpLine);
			if(!findDirBefore(tmpLine)){	//search as dir
				changed = true;
				recurseDirs(tmpDir);
			}
			 else{		//dir. was found (It isn't new one)
			 	AbeforeFile->setStartToCur();	//start for dir
			 	AbeforeFile->setBreakToCur();	//start for files
			 }
		}
		 else{		//another file
	 	 	int curPos = 0;
	 	 	if(!findInBefore(tmpLine, &curPos)){		//search as file
	 	 		changed = true;
#ifdef _DEBUG
				printf("INNONEW - %s INDIR - %s\n",tmpLine, tmpDir );
#endif	 	 		
	 	 		rmFile(tmpDir, tmpLine, true);		//write as remove		 	 	
		 	}
		  	else{
		  		if(curPos > partPos){
		  			partPos = curPos;
		  			AbeforeFile->setStartToCur();
		  		}
		  	}
		 }
	}
#ifdef _DEBUG
	printf("CompareFiles: IsNew - %d\n", changed);
#endif
	return((changed) ? 0 : 20);
}

bool Uninst::findDirBefore(const char* aftChar)	//search aftChar in afterFile (true = found)
{
	AbeforeFile->goToStart();
	const char* tmpLine;
	
	while(!AbeforeFile->eof()){
		tmpLine = AbeforeFile->readLine();
		if(!strcmp( aftChar, tmpLine )){
			return(true);		//path found!
		}
	}
	return(false);
}

bool Uninst::findInBefore(const char* aftChar, int* curPos)	//search aftChar in afterFile (true = found)
{
 	AbeforeFile->goToBreak();
	const char* tmpLine;
	
	while(!AbeforeFile->eof()){
		tmpLine = AbeforeFile->readLine();
		if((*tmpLine) == '/'){
			return(false);
		}
		(*curPos)++;
		if(!strcmp( aftChar, tmpLine )){
			return(true);		//path found!
		}
	}
	AbeforeFile->setStartToCur();
	return(false);
}


void Uninst::rmFile(const char* folder, const char* file, bool remove)
{
	char* sForce = "rm -f  \"";
	char* sNoForce = "rm -i  \"";
	char* sComment = "#  \"";
	char* select;
	char tmpLine[SIZE_BUFF];
	
	if(!remove){
		select = sComment;
	}
	 else if(conf->getForce()==SET_ON){
		select = sForce;
	 }
	  else{
	 	select = sNoForce;
	  }
	uninstFile->writeLine(select);
	strcpy(tmpLine, folder);
	*(tmpLine+strlen(tmpLine)-1)='\0';
	strcat(tmpLine, "/");
	strcat(tmpLine, file);
	*(tmpLine+strlen(tmpLine)-1) = '\0';
	uninstFile->writeLine(tmpLine);
	uninstFile->writeLine("\"\n");
}


void Uninst::rmDir(const char* folder)
{
	uninstFile->writeLine("rmdir  \"");
	char sLine[SIZE_BUFF];
	strcpy(sLine, folder);
	*(sLine+strlen(folder)-1) = '\0';
	uninstFile->writeLine(sLine);
	uninstFile->writeLine("\"\n");
}
	
void Uninst::recurseDirs(const char* parentDir)
{
	const char* tmpLine;
	char tmpDir[SIZE_BUFF];
	strcpy(tmpDir, "-fatal-");
	
	long int prevPos = AafterFile->getPos();
	
	while(!AafterFile->eof()){
		 tmpLine = AafterFile->readLine();
		
		if(*tmpLine=='/'){	//another sub dir
			if( (!strncmp(parentDir, tmpLine, strlen(parentDir)-1))
				     && (*(tmpLine+strlen(parentDir)-1)=='/') ){
				strcpy(tmpDir, tmpLine);
				recurseDirs(tmpDir);
				prevPos = AafterFile->getPos();
			}
			 else{		//another dir
				AafterFile->setPos(prevPos);
				break;	
			 }
			
		}
		 else{		//another file (in new dir)
			prevPos = AafterFile->getPos();
 	 		rmFile(parentDir, tmpLine, true);		//write as remove
		 }
	}
#ifdef _DEBUG
	printf("Delete - %s", parentDir);
#endif
	rmDir(parentDir);
	return;
}