#include "writetextfile.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>


WriteTextFile::WriteTextFile(const char* nameString): nameIs(false), inUse(false)
{
	if( nameString == 0 ){
 		errorNum=2;		// pointer is invalid
		return;
	}
	
	fileName = (char*)calloc( sizeof(char), strlen(nameString) + 1);
	if( fileName == NULL){
		errorNum=3;		// memory error;
		return;
	}
	nameIs=true;
	
	(void) strcpy( fileName, nameString);
#ifdef _DEBUG
	printf("WriteTextFile: file = %s\n", fileName);
#endif	
	fw = fopen( fileName, "w");
	if( fw == NULL ){
		errorNum=2;		// unable to open(create) file
		return;
	}	
	inUse=true;
		
	errorNum=0;
}

WriteTextFile::~WriteTextFile()
{
	if(inUse) (void)fclose(fw);
	if(nameIs) free(fileName);
#ifdef _DEBUG
	printf("~WriteTextFile: Deleting object...\n");
#endif
}

void WriteTextFile::writeLine(const char* line)
{
	if( fputs( line, fw) == EOF){
		errorNum=13;		// unable to write a line to file
		return;
	}
	errorNum=0;
	return;
}


void WriteTextFile::close()
{
	if(inUse == false){
		errorNum=14;
		return;
	}

	if(fw == NULL){
		errorNum=14;
		return;
	}
	if(fclose(fw) == EOF){
		errorNum=14;		// unable to close file
		return;
	}
	fw=NULL;
	inUse=false;
	errorNum=0;
#ifdef _DEBUG
	printf("WriteTextFile::close:	File closed...\n");
#endif
	return;
}

void WriteTextFile::setRunBit()
{
	if(chmod( fileName, mode_t (S_IRUSR | S_IWUSR | S_IXUSR)) ){
		errorNum=97;
	}
	  else{
	  	errorNum=0;
	  }
	return;
}

const char* WriteTextFile::getName() const
{
	if(nameIs){
		return(fileName);
	}
	return((char*)NULL);
}

void WriteTextFile::deleteFile()
{
	(void) fclose(fw);
	
	fw=NULL;
	inUse=false;
	
	if( (remove( fileName )) == -1){
		errorNum=8;		// can't delete file
		return;
	}
	errorNum=0;
	return;
}